@@ -23,9 +23,7 @@
 
 #include "uvm_common.h"
 #include "uvm_linux.h"
-#include "uvm_linux_ioctl.h"
 #include "uvm8_gpu.h"
-#include "uvm8_init.h"
 #include "uvm8_lock.h"
 #include "uvm8_va_space.h"
 #include "uvm8_va_range.h"
@@ -119,7 +117,7 @@ typedef struct
     // before calling migrate_vma.
     struct {
         DECLARE_BITMAP(page_mask, UVM_MIGRATE_VMA_MAX_PAGES);
-    } processors[UVM_MAX_PROCESSORS];
+    } processors[UVM_ID_MAX_PROCESSORS];
 
     // Number of pages in the migrate_vma call
     unsigned long num_pages;
@@ -137,10 +135,13 @@ static NV_STATUS migrate_vma_page_copy_address(struct page *page,
                                                migrate_vma_state_t *state,
                                                uvm_gpu_address_t *gpu_addr)
 {
-    uvm_gpu_t *owning_gpu = (resident_id == UVM_CPU_ID)? NULL: uvm_gpu_get(resident_id);
+    uvm_va_space_t *va_space = state->va_space;
+    uvm_gpu_t *owning_gpu = UVM_ID_IS_CPU(resident_id)? NULL: uvm_va_space_get_gpu(va_space, resident_id);
+    const bool can_copy_from = uvm_processor_mask_test(&va_space->can_copy_from[uvm_id_value(copying_gpu->id)],
+                                                       resident_id);
     const bool direct_peer = owning_gpu &&
                              (owning_gpu != copying_gpu) &&
-                             uvm_processor_mask_test(&state->va_space->can_copy_from[copying_gpu->id], resident_id) &&
+                             can_copy_from &&
                              !uvm_gpu_peer_caps(owning_gpu, copying_gpu)->is_indirect_peer;
 
     UVM_ASSERT(page_index < state->num_pages);
@@ -153,10 +154,10 @@ static NV_STATUS migrate_vma_page_copy_address(struct page *page,
     }
     else if (direct_peer) {
         // Direct GPU peer
-        NvU32 peer_index = uvm_gpu_index(owning_gpu->id);
+        uvm_gpu_identity_mapping_t *gpu_peer_mappings = uvm_gpu_get_peer_mapping(copying_gpu, owning_gpu->id);
         uvm_gpu_phys_address_t phys_addr = uvm_gpu_page_to_phys_address(owning_gpu, page);
 
-        *gpu_addr = uvm_gpu_address_virtual(copying_gpu->peer_mappings[peer_index].base + phys_addr.address);
+        *gpu_addr = uvm_gpu_address_virtual(gpu_peer_mappings->base + phys_addr.address);
     }
     else {
         // Sysmem/Indirect Peer
@@ -203,11 +204,11 @@ static NV_STATUS migrate_vma_zero_begin_push(uvm_va_space_t *va_space,
 {
     uvm_channel_type_t channel_type;
 
-    if (dst_id == UVM_CPU_ID) {
+    if (UVM_ID_IS_CPU(dst_id)) {
         channel_type = UVM_CHANNEL_TYPE_GPU_TO_CPU;
     }
     else {
-        UVM_ASSERT(dst_id == gpu->id);
+        UVM_ASSERT(uvm_id_equal(dst_id, gpu->id));
         channel_type = UVM_CHANNEL_TYPE_GPU_INTERNAL;
     }
 
@@ -215,8 +216,8 @@ static NV_STATUS migrate_vma_zero_begin_push(uvm_va_space_t *va_space,
                           channel_type,
                           push,
                           "Zero %s from %s VMA region [0x%lx, 0x%lx]",
-                          uvm_processor_name(dst_id),
-                          uvm_processor_name(gpu->id),
+                          uvm_va_space_processor_name(va_space, dst_id),
+                          uvm_va_space_processor_name(va_space, gpu->id),
                           start,
                           outer);
 }
@@ -232,20 +233,22 @@ static NV_STATUS migrate_vma_copy_begin_push(uvm_va_space_t *va_space,
     uvm_channel_type_t channel_type;
     uvm_gpu_t *gpu;
 
-    UVM_ASSERT_MSG(src_id != dst_id, "Unexpected copy to self, processor %s\n", uvm_processor_name(src_id));
+    UVM_ASSERT_MSG(!uvm_id_equal(src_id, dst_id),
+                   "Unexpected copy to self, processor %s\n",
+                   uvm_va_space_processor_name(va_space, src_id));
 
-    if (src_id == UVM_CPU_ID) {
-        gpu = uvm_gpu_get(dst_id);
+    if (UVM_ID_IS_CPU(src_id)) {
+        gpu = uvm_va_space_get_gpu(va_space, dst_id);
         channel_type = UVM_CHANNEL_TYPE_CPU_TO_GPU;
     }
-    else if (dst_id == UVM_CPU_ID) {
-        gpu = uvm_gpu_get(src_id);
+    else if (UVM_ID_IS_CPU(dst_id)) {
+        gpu = uvm_va_space_get_gpu(va_space, src_id);
         channel_type = UVM_CHANNEL_TYPE_GPU_TO_CPU;
     }
     else {
         // For GPU to GPU copies, prefer to "push" the data from the source as
         // that works better
-        gpu = uvm_gpu_get(src_id);
+        gpu = uvm_va_space_get_gpu(va_space, src_id);
 
         channel_type = UVM_CHANNEL_TYPE_GPU_TO_GPU;
     }
@@ -254,26 +257,26 @@ static NV_STATUS migrate_vma_copy_begin_push(uvm_va_space_t *va_space,
     // P2P access has not been explicitly enabled (ie va_space->can_copy_from
     // is not set).
     if (!gpu->numa_info.enabled) {
-        UVM_ASSERT_MSG(uvm_processor_mask_test(&va_space->can_copy_from[gpu->id], dst_id),
+        UVM_ASSERT_MSG(uvm_processor_mask_test(&va_space->can_copy_from[uvm_id_value(gpu->id)], dst_id),
                        "GPU %s dst %s src %s\n",
-                       uvm_processor_name(gpu->id),
-                       uvm_processor_name(dst_id),
-                       uvm_processor_name(src_id));
-        UVM_ASSERT_MSG(uvm_processor_mask_test(&va_space->can_copy_from[gpu->id], src_id),
+                       uvm_va_space_processor_name(va_space, gpu->id),
+                       uvm_va_space_processor_name(va_space, dst_id),
+                       uvm_va_space_processor_name(va_space, src_id));
+        UVM_ASSERT_MSG(uvm_processor_mask_test(&va_space->can_copy_from[uvm_id_value(gpu->id)], src_id),
                        "GPU %s dst %s src %s\n",
-                       uvm_processor_name(gpu->id),
-                       uvm_processor_name(dst_id),
-                       uvm_processor_name(src_id));
+                       uvm_va_space_processor_name(va_space, gpu->id),
+                       uvm_va_space_processor_name(va_space, dst_id),
+                       uvm_va_space_processor_name(va_space, src_id));
     }
 
     if (channel_type == UVM_CHANNEL_TYPE_GPU_TO_GPU) {
-        uvm_gpu_t *dst_gpu = uvm_gpu_get(dst_id);
+        uvm_gpu_t *dst_gpu = uvm_va_space_get_gpu(va_space, dst_id);
         return uvm_push_begin_gpu_to_gpu(gpu->channel_manager,
                                          dst_gpu,
                                          push,
                                          "Copy from %s to %s for VMA region [0x%lx, 0x%lx]",
-                                         uvm_processor_name(src_id),
-                                         uvm_processor_name(dst_id),
+                                         uvm_va_space_processor_name(va_space, src_id),
+                                         uvm_va_space_processor_name(va_space, dst_id),
                                          start,
                                          outer);
     }
@@ -282,8 +285,8 @@ static NV_STATUS migrate_vma_copy_begin_push(uvm_va_space_t *va_space,
                           channel_type,
                           push,
                           "Copy from %s to %s for VMA region [0x%lx, 0x%lx]",
-                          uvm_processor_name(src_id),
-                          uvm_processor_name(dst_id),
+                          uvm_va_space_processor_name(va_space, src_id),
+                          uvm_va_space_processor_name(va_space, dst_id),
                           start,
                           outer);
 }
@@ -332,9 +335,9 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
                 // Populate PROT_WRITE vmas in migrate_vma so we can use the
                 // GPU's copy engines
                 if (state->num_populate_anon_pages++ == 0)
-                    bitmap_zero(state->processors[state->dst_id].page_mask, state->num_pages);
+                    bitmap_zero(state->processors[uvm_id_value(state->dst_id)].page_mask, state->num_pages);
 
-                __set_bit(i, state->processors[state->dst_id].page_mask);
+                __set_bit(i, state->processors[uvm_id_value(state->dst_id)].page_mask);
             }
             else {
                 __clear_bit(i, state->pending_page_mask);
@@ -359,7 +362,7 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
         }
 
         // Already resident on a CPU node, don't move
-        if (state->dst_id == UVM_CPU_ID && node_state(src_nid, N_CPU)) {
+        if (UVM_ID_IS_CPU(state->dst_id) && node_state(src_nid, N_CPU)) {
             __clear_bit(i, state->pending_page_mask);
             continue;
         }
@@ -368,7 +371,7 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
 
         // Already resident on a node with no CPUs that doesn't belong to a
         // GPU, don't move
-        if (state->dst_id == UVM_CPU_ID && !src_gpu) {
+        if (UVM_ID_IS_CPU(state->dst_id) && !src_gpu) {
             __clear_bit(i, state->pending_page_mask);
             continue;
         }
@@ -380,12 +383,12 @@ static void migrate_vma_compute_masks(struct vm_area_struct *vma,
         if (src_gpu)
             src_id = src_gpu->id;
         else
-            src_id = UVM_CPU_ID;
+            src_id = UVM_ID_CPU;
 
         if (!uvm_processor_mask_test_and_set(&state->src_processors, src_id))
-            bitmap_zero(state->processors[src_id].page_mask, state->num_pages);
+            bitmap_zero(state->processors[uvm_id_value(src_id)].page_mask, state->num_pages);
 
-        __set_bit(i, state->processors[src_id].page_mask);
+        __set_bit(i, state->processors[uvm_id_value(src_id)].page_mask);
     }
 }
 
@@ -424,7 +427,7 @@ static NV_STATUS migrate_vma_populate_anon_pages(struct vm_area_struct *vma,
 {
     NV_STATUS status = NV_OK;
     unsigned long i;
-    unsigned long *page_mask = state->processors[state->dst_id].page_mask;
+    unsigned long *page_mask = state->processors[uvm_id_value(state->dst_id)].page_mask;
     uvm_push_t push;
     uvm_gpu_t *copying_gpu = NULL;
     uvm_va_space_t *va_space = state->va_space;
@@ -451,13 +454,13 @@ static NV_STATUS migrate_vma_populate_anon_pages(struct vm_area_struct *vma,
         if (!copying_gpu) {
             // Try to get a GPU attached to the node being populated. If there
             // is none, use any of the GPUs registered in the VA space.
-            if (state->dst_id == UVM_CPU_ID) {
+            if (UVM_ID_IS_CPU(state->dst_id)) {
                 copying_gpu = uvm_va_space_find_first_gpu_attached_to_cpu_node(va_space, state->dst_node_id);
                 if (!copying_gpu)
-                    copying_gpu = uvm_processor_mask_find_first_gpu(&va_space->registered_gpus);
+                    copying_gpu = uvm_va_space_find_first_gpu(va_space);
             }
             else {
-                copying_gpu = uvm_gpu_get(state->dst_id);
+                copying_gpu = uvm_va_space_get_gpu(va_space, state->dst_id);
             }
 
             UVM_ASSERT(copying_gpu);
@@ -512,7 +515,7 @@ static NV_STATUS migrate_vma_copy_pages_from(struct vm_area_struct *vma,
     uvm_push_t push;
     uvm_gpu_t *copying_gpu = NULL;
     unsigned long i;
-    unsigned long *page_mask = state->processors[src_id].page_mask;
+    unsigned long *page_mask = state->processors[uvm_id_value(src_id)].page_mask;
     uvm_va_space_t *va_space = state->va_space;
 
     UVM_ASSERT(!bitmap_empty(page_mask, state->num_pages));
@@ -799,7 +802,7 @@ static NV_STATUS migrate_pageable_vma_region(struct vm_area_struct *vma,
         // If the destination is the CPU, signal user-space to retry with a
         // different node. Otherwise, just try to populate anywhere in the
         // system
-        if (migrate_vma_state->dst_id == UVM_CPU_ID) {
+        if (UVM_ID_IS_CPU(migrate_vma_state->dst_id)) {
             *next_addr = start + find_first_bit(migrate_vma_state->scratch2_mask, num_pages) * PAGE_SIZE;
             return NV_ERR_MORE_PROCESSING_REQUIRED;
         }
@@ -909,7 +912,7 @@ static NV_STATUS migrate_pageable(struct mm_struct *mm,
         else if (status == NV_ERR_MORE_PROCESSING_REQUIRED) {
             UVM_ASSERT(next_addr >= start);
             UVM_ASSERT(next_addr < outer);
-            UVM_ASSERT(migrate_vma_state->dst_id == UVM_CPU_ID);
+            UVM_ASSERT(UVM_ID_IS_CPU(migrate_vma_state->dst_id));
 
             *user_space_start = next_addr;
         }
@@ -949,8 +952,8 @@ NV_STATUS uvm_migrate_pageable(uvm_va_space_t *va_space,
     // uvm_migrate_pageable is only called from the process context
     // (uvm_migrate). However, this would need to change if we wanted to call
     // this function from a bottom half.
-    if (dst_id == UVM_CPU_ID &&
-        (!uvm_memory_node_is_valid(dst_cpu_node_id) || get_gpu_from_node_id(va_space, dst_cpu_node_id) != NULL))
+    if (UVM_ID_IS_CPU(dst_id) &&
+        (!nv_numa_node_has_memory(dst_cpu_node_id) || get_gpu_from_node_id(va_space, dst_cpu_node_id) != NULL))
         return NV_ERR_INVALID_ARGUMENT;
 
     migrate_vma_state = kmem_cache_alloc(g_uvm_migrate_vma_state_cache, NV_UVM_GFP_FLAGS);
@@ -959,8 +962,9 @@ NV_STATUS uvm_migrate_pageable(uvm_va_space_t *va_space,
 
     migrate_vma_state->va_space    = va_space;
     migrate_vma_state->dst_id      = dst_id;
-    migrate_vma_state->dst_node_id = (dst_id == UVM_CPU_ID)? dst_cpu_node_id :
-                                                             uvm_gpu_numa_info(uvm_gpu_get(dst_id))->node_id;
+    migrate_vma_state->dst_node_id = UVM_ID_IS_CPU(dst_id)?
+                                         dst_cpu_node_id :
+                                         uvm_gpu_numa_info(uvm_va_space_get_gpu(va_space, dst_id))->node_id;
 
     status = migrate_pageable(mm, start, length, migrate_vma_state, user_space_start, user_space_length);
 
