@@ -33,11 +33,13 @@
 #include "uvm8_mmu.h"
 #include "uvm_linux.h"
 #include "nv-kref.h"
+#include "nv-linux.h"
 #include "uvm8_perf_events.h"
 #include "uvm8_perf_module.h"
 #include "uvm8_va_block_types.h"
 #include "uvm8_va_block.h"
 #include "uvm8_hmm.h"
+#include "uvm8_test_ioctl.h"
 
 // uvm_deferred_free_object provides a mechanism for building and later freeing
 // a list of objects which are owned by a VA space, but can't be freed while the
@@ -161,11 +163,29 @@ struct uvm_gpu_va_space_struct
     } ats;
 };
 
+typedef struct
+{
+    int                  numa_node;
+
+    uvm_processor_mask_t gpus;
+} uvm_cpu_gpu_affinity_t;
+
 struct uvm_va_space_struct
 {
     // Mask of gpus registered with the va space
     uvm_processor_mask_t registered_gpus;
 
+    // Array of pointers to the uvm_gpu_t objects that correspond to the
+    // uvm_processor_id_t index.
+
+
+
+
+
+
+
+    uvm_gpu_t *registered_gpus_table[UVM_ID_MAX_GPUS];
+
     // Mask of processors registered with the va space that support replayable faults
     uvm_processor_mask_t faultable_processors;
 
@@ -218,38 +238,38 @@ struct uvm_va_space_struct
     // which that processor can directly access. In other words, this will test
     // whether A has direct access to B:
     //      uvm_processor_mask_test(can_access[A], B)
-    uvm_processor_mask_t can_access[UVM_MAX_PROCESSORS];
+    uvm_processor_mask_t can_access[UVM_ID_MAX_PROCESSORS];
 
     // Pre-computed masks that contain, for each processor memory, a mask with
     // the processors that have direct access enabled to its memory. This is the
     // opposite direction as can_access. In other words, this will test whether
     // A has direct access to B:
     //      uvm_processor_mask_test(accessible_from[B], A)
-    uvm_processor_mask_t accessible_from[UVM_MAX_PROCESSORS];
+    uvm_processor_mask_t accessible_from[UVM_ID_MAX_PROCESSORS];
 
     // Pre-computed masks that contain, for each processor memory, a mask with
     // the processors that can directly copy to and from its memory. This is
     // almost the same as accessible_from masks, but also requires peer identity
     // mappings to be supported for peer access.
-    uvm_processor_mask_t can_copy_from[UVM_MAX_PROCESSORS];
+    uvm_processor_mask_t can_copy_from[UVM_ID_MAX_PROCESSORS];
 
     // Pre-computed masks that contain, for each processor, a mask of processors
     // to which that processor has NVLINK access. In other words, this will test
     // whether A has NVLINK access to B:
     //      uvm_processor_mask_test(has_nvlink[A], B)
     // This is a subset of can_access.
-    uvm_processor_mask_t has_nvlink[UVM_MAX_PROCESSORS];
+    uvm_processor_mask_t has_nvlink[UVM_ID_MAX_PROCESSORS];
 
     // Pre-computed masks that contain, for each processor memory, a mask with
     // the processors that have direct access to its memory and native support
     // for atomics in HW. This is a subset of accessible_from.
-    uvm_processor_mask_t has_native_atomics[UVM_MAX_PROCESSORS];
+    uvm_processor_mask_t has_native_atomics[UVM_ID_MAX_PROCESSORS];
 
     // Pre-computed masks that contain, for each processor memory, a mask with
     // the processors that are indirect peers. Indirect peers can access each
     // other's memory like regular peers, but with additional latency and/or bw
     // penalty.
-    uvm_processor_mask_t indirect_peers[UVM_MAX_PROCESSORS];
+    uvm_processor_mask_t indirect_peers[UVM_ID_MAX_PROCESSORS];
 
     // Mask of gpu_va_spaces registered with the va space
     // indexed by gpu->id
@@ -271,17 +291,12 @@ struct uvm_va_space_struct
     // as new CPU NUMA nodes are discovered on GPU registration. Each entry
     // contains a CPU NUMA node id, and a mask with the GPUs attached to it.
     // Since each GPU can only be attached to one CPU node id, the array can
-    // contain information for up to UVM_MAX_GPUS nodes. The information is
+    // contain information for up to UVM_ID_MAX_GPUS nodes. The information is
     // stored in the VA space to avoid taking the global lock.
-    struct
-    {
-        int                  numa_node;
-
-        uvm_processor_mask_t gpus;
-    } cpu_gpu_numa_affinity[UVM_MAX_GPUS];
+    uvm_cpu_gpu_affinity_t gpu_cpu_numa_affinity[UVM_ID_MAX_GPUS];
 
     // Array of GPU VA spaces
-    uvm_gpu_va_space_t *gpu_va_spaces[UVM_MAX_GPUS];
+    uvm_gpu_va_space_t *gpu_va_spaces[UVM_ID_MAX_GPUS];
 
     // Tracking of GPU VA spaces which have dropped the VA space lock and are
     // pending destruction. uvm_va_space_mm_shutdown has to wait for those
@@ -363,12 +378,57 @@ struct uvm_va_space_struct
         bool  page_prefetch_enabled;
 
         atomic_t migrate_vma_allocation_fail_nth;
+
+        uvm_thread_context_wrapper_t *dummy_thread_context_wrappers;
+        size_t num_dummy_thread_context_wrappers;
     } test;
 
     // Queue item for deferred f_ops->release() handling
     nv_kthread_q_item_t deferred_release_q_item;
 };
 
+static uvm_gpu_t *uvm_va_space_get_gpu(uvm_va_space_t *va_space, uvm_gpu_id_t gpu_id)
+{
+    uvm_gpu_t *gpu;
+
+    UVM_ASSERT(uvm_processor_mask_test(&va_space->registered_gpus, gpu_id));
+
+    gpu = va_space->registered_gpus_table[uvm_id_gpu_index(gpu_id)];
+
+    UVM_ASSERT(gpu);
+    UVM_ASSERT(uvm_gpu_get(gpu->global_id) == gpu);
+
+    return gpu;
+}
+
+static const char *uvm_va_space_processor_name(uvm_va_space_t *va_space, uvm_processor_id_t id)
+{
+    if (UVM_ID_IS_CPU(id))
+        return "0: CPU";
+    else
+        return uvm_va_space_get_gpu(va_space, id)->name;
+}
+
+static void uvm_va_space_processor_uuid(uvm_va_space_t *va_space, NvProcessorUuid *uuid, uvm_processor_id_t id)
+{
+    if (UVM_ID_IS_CPU(id)) {
+        memcpy(uuid, &NV_PROCESSOR_UUID_CPU_DEFAULT, sizeof(*uuid));
+    }
+    else {
+        uvm_gpu_t *gpu = uvm_va_space_get_gpu(va_space, id);
+        UVM_ASSERT(gpu);
+        memcpy(uuid, &gpu->uuid, sizeof(*uuid));
+    }
+}
+
+static bool uvm_va_space_processor_has_memory(uvm_va_space_t *va_space, uvm_processor_id_t id)
+{
+    if (UVM_ID_IS_CPU(id))
+        return true;
+
+    return uvm_va_space_get_gpu(va_space, id)->mem_info.size > 0;
+}
+
 NV_STATUS uvm_va_space_create(struct inode *inode, struct file *filp);
 void uvm_va_space_destroy(uvm_va_space_t *va_space);
 
@@ -563,28 +623,120 @@ static uvm_gpu_va_space_t *uvm_gpu_va_space_get(uvm_va_space_t *va_space, uvm_gp
     if (!gpu || !uvm_processor_mask_test(&va_space->registered_gpu_va_spaces, gpu->id))
         return NULL;
 
-    gpu_va_space = va_space->gpu_va_spaces[uvm_gpu_index(gpu->id)];
+    gpu_va_space = va_space->gpu_va_spaces[uvm_id_gpu_index(gpu->id)];
     UVM_ASSERT(uvm_gpu_va_space_state(gpu_va_space) == UVM_GPU_VA_SPACE_STATE_ACTIVE);
     UVM_ASSERT(gpu_va_space->va_space == va_space);
     UVM_ASSERT(gpu_va_space->gpu == gpu);
     return gpu_va_space;
 }
 
-#define for_each_gpu_va_space(__gpu_va_space, __va_space)                                               \
-    for (__gpu_va_space = uvm_gpu_va_space_get(                                                         \
-                              __va_space,                                                               \
-                              uvm_processor_mask_find_first_gpu(&__va_space->registered_gpu_va_spaces)  \
-                          );                                                                            \
-         __gpu_va_space;                                                                                \
-         __gpu_va_space = uvm_gpu_va_space_get(                                                         \
-                              __va_space,                                                               \
-                              uvm_processor_mask_find_next_gpu(&__va_space->registered_gpu_va_spaces,   \
-                                                               __gpu_va_space->gpu)                     \
-                          )                                                                             \
+#define for_each_gpu_va_space(__gpu_va_space, __va_space)                                                   \
+    for (__gpu_va_space =                                                                                   \
+            uvm_gpu_va_space_get(                                                                           \
+                __va_space,                                                                                 \
+                uvm_processor_mask_find_first_va_space_gpu(&__va_space->registered_gpu_va_spaces, va_space) \
+            );                                                                                              \
+         __gpu_va_space;                                                                                    \
+         __gpu_va_space =                                                                                   \
+            uvm_gpu_va_space_get(                                                                           \
+                __va_space,                                                                                 \
+                __uvm_processor_mask_find_next_va_space_gpu(&__va_space->registered_gpu_va_spaces,          \
+                                                            va_space,                                       \
+                                                            __gpu_va_space->gpu)                            \
+            )                                                                                               \
         )
 
+// Return the first GPU set in the given mask or NULL. The caller must ensure
+// that the GPUs set in the mask are registered in the VA space and cannot be
+// unregistered during this call.
+static uvm_gpu_t *uvm_processor_mask_find_first_va_space_gpu(const uvm_processor_mask_t *mask, uvm_va_space_t *va_space)
+{
+    uvm_gpu_t *gpu;
+    uvm_gpu_id_t gpu_id;
+
+    UVM_ASSERT(uvm_processor_mask_subset(mask, &va_space->registered_gpus));
+
+    gpu_id = uvm_processor_mask_find_first_gpu_id(mask);
+    if (UVM_ID_IS_INVALID(gpu_id))
+        return NULL;
+
+    gpu = uvm_va_space_get_gpu(va_space, gpu_id);
+    UVM_ASSERT_MSG(gpu, "gpu_id %u\n", uvm_id_value(gpu_id));
+
+    return gpu;
+}
+
+static uvm_gpu_t *uvm_va_space_find_first_gpu(uvm_va_space_t *va_space)
+{
+    uvm_assert_rwsem_locked(&va_space->lock);
+
+    return uvm_processor_mask_find_first_va_space_gpu(&va_space->registered_gpus, va_space);
+}
+
+// Same as uvm_processor_mask_find_next_va_space_gpu below, but gpu cannot be
+// NULL
+static uvm_gpu_t *__uvm_processor_mask_find_next_va_space_gpu(const uvm_processor_mask_t *mask,
+                                                              uvm_va_space_t *va_space,
+                                                              uvm_gpu_t *gpu)
+{
+    uvm_gpu_id_t gpu_id;
+
+    UVM_ASSERT(gpu != NULL);
+    UVM_ASSERT(uvm_processor_mask_subset(mask, &va_space->registered_gpus));
+
+    gpu_id = uvm_processor_mask_find_next_id(mask, uvm_gpu_id_next(gpu->id));
+    if (UVM_ID_IS_INVALID(gpu_id))
+        return NULL;
+
+    gpu = uvm_va_space_get_gpu(va_space, gpu_id);
+    UVM_ASSERT_MSG(gpu, "gpu_id %u\n", uvm_id_value(gpu_id));
+
+    return gpu;
+}
+
+// Return the next GPU with an id larger than gpu->id set in the given mask.
+// The function returns NULL if gpu is NULL. The caller must ensure that the
+// GPUs set in the mask are registered in the VA space and cannot be
+// unregistered during this call.
+static uvm_gpu_t *uvm_processor_mask_find_next_va_space_gpu(const uvm_processor_mask_t *mask,
+                                                            uvm_va_space_t *va_space,
+                                                            uvm_gpu_t *gpu)
+{
+    if (gpu == NULL)
+        return NULL;
+
+    return __uvm_processor_mask_find_next_va_space_gpu(mask, va_space, gpu);
+}
+
+#define for_each_va_space_gpu_in_mask(gpu, va_space, mask)                                       \
+    for (({uvm_assert_rwsem_locked(&(va_space)->lock);                                           \
+           gpu = uvm_processor_mask_find_first_va_space_gpu(mask, va_space);});                  \
+           gpu != NULL;                                                                          \
+           gpu = __uvm_processor_mask_find_next_va_space_gpu(mask, va_space, gpu))
+
+// Helper to iterate over all GPUs registered in a UVM VA space
+#define for_each_va_space_gpu(gpu, va_space) \
+    for_each_va_space_gpu_in_mask(gpu, va_space, &(va_space)->registered_gpus)
+
+static void uvm_va_space_global_gpus_in_mask(uvm_va_space_t *va_space,
+                                             uvm_global_processor_mask_t *global_mask,
+                                             const uvm_processor_mask_t *mask)
+{
+    uvm_gpu_t *gpu;
+
+    uvm_global_processor_mask_zero(global_mask);
+
+    for_each_va_space_gpu_in_mask(gpu, va_space, mask)
+        uvm_global_processor_mask_set(global_mask, gpu->global_id);
+}
+
+static void uvm_va_space_global_gpus(uvm_va_space_t *va_space, uvm_global_processor_mask_t *global_mask)
+{
+    uvm_va_space_global_gpus_in_mask(va_space, global_mask, &va_space->registered_gpus);
+}
+
 // Return the processor in the candidates mask that is "closest" to src, or
-// UVM_MAX_PROCESSORS if candidates is empty. The order is:
+// UVM_ID_MAX_PROCESSORS if candidates is empty. The order is:
 // - src itself
 // - Direct NVLINK GPU peers if src is CPU or GPU (1)
 // - NVLINK CPU if src is GPU
@@ -607,7 +759,7 @@ uvm_processor_id_t uvm_processor_mask_find_closest_id(uvm_va_space_t *va_space,
 // destructive to mask.
 #define for_each_closest_id(id, mask, src, va_space)                    \
     for (id = uvm_processor_mask_find_closest_id(va_space, mask, src);  \
-         id != UVM_MAX_PROCESSORS;                                     \
+         UVM_ID_IS_VALID(id);                                           \
          uvm_processor_mask_clear(mask, id), id = uvm_processor_mask_find_closest_id(va_space, mask, src))
 
 // Return the GPU whose memory corresponds to the given node_id
@@ -615,7 +767,7 @@ static uvm_gpu_t *uvm_va_space_find_gpu_with_memory_node_id(uvm_va_space_t *va_s
 {
     uvm_gpu_t *gpu;
 
-    UVM_ASSERT(uvm_memory_node_is_valid(node_id));
+    UVM_ASSERT(nv_numa_node_has_memory(node_id));
 
     if (!g_uvm_global.ats.supported)
         return NULL;
@@ -641,13 +793,14 @@ static void uvm_va_space_get_gpus_attached_to_cpu_node(uvm_va_space_t *va_space,
                                                        int node_id,
                                                        uvm_processor_mask_t *gpus)
 {
-    unsigned i;
+    uvm_gpu_id_t gpu_id;
 
     UVM_ASSERT(!uvm_va_space_memory_node_is_gpu(va_space, node_id));
 
-    for (i = 0; i < ARRAY_SIZE(va_space->cpu_gpu_numa_affinity); ++i) {
-        if (va_space->cpu_gpu_numa_affinity[i].numa_node == node_id) {
-            uvm_processor_mask_copy(gpus, &va_space->cpu_gpu_numa_affinity[i].gpus);
+    for_each_gpu_id(gpu_id) {
+        const uvm_cpu_gpu_affinity_t *affinity = &va_space->gpu_cpu_numa_affinity[uvm_id_gpu_index(gpu_id)];
+        if (affinity->numa_node == node_id) {
+            uvm_processor_mask_copy(gpus, &affinity->gpus);
             return;
         }
     }
@@ -663,7 +816,7 @@ static uvm_gpu_t *uvm_va_space_find_first_gpu_attached_to_cpu_node(uvm_va_space_
 
     uvm_va_space_get_gpus_attached_to_cpu_node(va_space, node_id, &gpus);
 
-    return uvm_processor_mask_find_first_gpu(&gpus);
+    return uvm_processor_mask_find_first_va_space_gpu(&gpus, va_space);
 }
 
 // Obtain the user channel with the given instance_ptr. This is used during
